@@ -1,6 +1,6 @@
 ;;; git-rebase.el --- Edit Git rebase files  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2010-2016  The Magit Project Contributors
+;; Copyright (C) 2010-2017  The Magit Project Contributors
 ;;
 ;; You should have received a copy of the AUTHORS.md file which
 ;; lists all contributors.  If not, see http://magit.vc/authors.
@@ -83,6 +83,7 @@
 
 (defgroup git-rebase nil
   "Edit Git rebase sequences."
+  :link '(info-link "(magit)Editing Rebase Sequences")
   :group 'tools)
 
 (defcustom git-rebase-auto-advance t
@@ -127,7 +128,7 @@
 
 (defface git-rebase-comment-heading
   '((t :inherit font-lock-keyword-face))
-  "Face used for headings in rebase message comments."
+  "Face for headings in rebase message comments."
   :group 'git-commit-faces)
 
 ;;; Keymaps
@@ -135,35 +136,49 @@
 (defvar git-rebase-mode-map
   (let ((map (make-sparse-keymap)))
     (set-keymap-parent map special-mode-map)
-    (define-key map (kbd "q")    'undefined)
-    (define-key map [remap undo] 'git-rebase-undo)
-    (define-key map (kbd "RET") 'git-rebase-show-commit)
-    (define-key map (kbd "SPC") 'git-rebase-show-or-scroll-up)
-    (define-key map (kbd "DEL") 'git-rebase-show-or-scroll-down)
-    (define-key map (kbd "x")   'git-rebase-exec)
-    (define-key map (kbd "c")   'git-rebase-pick)
-    (define-key map (kbd "r")   'git-rebase-reword)
-    (define-key map (kbd "w")   'git-rebase-reword)
-    (define-key map (kbd "e")   'git-rebase-edit)
-    (define-key map (kbd "m")   'git-rebase-edit)
-    (define-key map (kbd "s")   'git-rebase-squash)
-    (define-key map (kbd "f")   'git-rebase-fixup)
-    (define-key map (kbd "y")   'git-rebase-insert)
-    (define-key map (kbd "k")   'git-rebase-kill-line)
-    (define-key map (kbd "C-k") 'git-rebase-kill-line)
-    (define-key map (kbd "p")   'git-rebase-backward-line)
-    (define-key map (kbd "n")   'forward-line)
-    (define-key map (kbd "M-p")      'git-rebase-move-line-up)
-    (define-key map (kbd "M-n")      'git-rebase-move-line-down)
-    (define-key map (kbd "M-<up>")   'git-rebase-move-line-up)
-    (define-key map (kbd "M-<down>") 'git-rebase-move-line-down)
-    (define-key map (kbd "C-x C-t")  'git-rebase-move-line-up)
+    (cond ((featurep 'jkl)
+           (define-key map [return]    'git-rebase-show-commit)
+           (define-key map (kbd   "i") 'git-rebase-backward-line)
+           (define-key map (kbd   "k") 'forward-line)
+           (define-key map (kbd "M-i") 'git-rebase-move-line-up)
+           (define-key map (kbd "M-k") 'git-rebase-move-line-down)
+           (define-key map (kbd   "p") 'git-rebase-pick)
+           (define-key map (kbd   ",") 'git-rebase-kill-line))
+          (t
+           (define-key map (kbd "C-m") 'git-rebase-show-commit)
+           (define-key map (kbd   "p") 'git-rebase-backward-line)
+           (define-key map (kbd   "n") 'forward-line)
+           (define-key map (kbd "M-p") 'git-rebase-move-line-up)
+           (define-key map (kbd "M-n") 'git-rebase-move-line-down)
+           (define-key map (kbd   "c") 'git-rebase-pick)
+           (define-key map (kbd   "k") 'git-rebase-kill-line)
+           (define-key map (kbd "C-k") 'git-rebase-kill-line)))
+    (define-key map (kbd "e") 'git-rebase-edit)
+    (define-key map (kbd "m") 'git-rebase-edit)
+    (define-key map (kbd "f") 'git-rebase-fixup)
+    (define-key map (kbd "q") 'undefined)
+    (define-key map (kbd "r") 'git-rebase-reword)
+    (define-key map (kbd "w") 'git-rebase-reword)
+    (define-key map (kbd "s") 'git-rebase-squash)
+    (define-key map (kbd "x") 'git-rebase-exec)
+    (define-key map (kbd "y") 'git-rebase-insert)
+    (define-key map (kbd "SPC")     'git-rebase-show-or-scroll-up)
+    (define-key map (kbd "DEL")     'git-rebase-show-or-scroll-down)
+    (define-key map (kbd "C-x C-t") 'git-rebase-move-line-up)
+    (define-key map [M-up]          'git-rebase-move-line-up)
+    (define-key map [M-down]        'git-rebase-move-line-down)
+    (define-key map [remap undo]    'git-rebase-undo)
     map)
   "Keymap for Git-Rebase mode.")
 
-(put 'git-rebase-reword :advertised-binding "r")
-(put 'git-rebase-move-line-up :advertised-binding (kbd "M-p"))
-(put 'git-rebase-kill-line :advertised-binding "k")
+(cond ((featurep 'jkl)
+       (put 'git-rebase-reword       :advertised-binding "r")
+       (put 'git-rebase-move-line-up :advertised-binding (kbd "M-i"))
+       (put 'git-rebase-kill-line    :advertised-binding ","))
+      (t
+       (put 'git-rebase-reword       :advertised-binding "r")
+       (put 'git-rebase-move-line-up :advertised-binding (kbd "M-p"))
+       (put 'git-rebase-kill-line    :advertised-binding "k")))
 
 (easy-menu-define git-rebase-mode-menu git-rebase-mode-map
   "Git-Rebase mode menu"
@@ -530,7 +545,4 @@ By default, this is the same except for the \"pick\" command."
 (add-to-list 'with-editor-file-name-history-exclude git-rebase-filename-regexp)
 
 (provide 'git-rebase)
-;; Local Variables:
-;; indent-tabs-mode: nil
-;; End:
 ;;; git-rebase.el ends here
