@@ -1,6 +1,6 @@
 ;;; magit-process.el --- process functionality  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2010-2016  The Magit Project Contributors
+;; Copyright (C) 2010-2017  The Magit Project Contributors
 ;;
 ;; You should have received a copy of the AUTHORS.md file which
 ;; lists all contributors.  If not, see http://magit.vc/authors.
@@ -337,12 +337,12 @@ conversion."
 
 (defun magit-cygwin-env-vars ()
   (append magit-git-environment
-          (when magit-need-cygwin-noglob
-            (mapcar (lambda (var)
-                      (concat var "=" (--if-let (getenv var)
-                                          (concat it " noglob")
-                                        "noglob")))
-                    '("CYGWIN" "MSYS")))))
+          (and magit-need-cygwin-noglob
+               (mapcar (lambda (var)
+                         (concat var "=" (--if-let (getenv var)
+                                             (concat it " noglob")
+                                           "noglob")))
+                       '("CYGWIN" "MSYS")))))
 
 (defvar magit-this-process nil)
 
@@ -595,7 +595,7 @@ Magit status buffer."
   "Special sentinel used by `magit-run-git-sequencer'."
   (when (memq (process-status process) '(exit signal))
     (magit-process-sentinel process event)
-    (-when-let (process-buf (process-get process 'process-buf))
+    (-when-let (process-buf (process-buffer process))
       (when (buffer-live-p process-buf)
         (-when-let (status-buf (with-current-buffer process-buf
                                  (magit-mode-get-buffer 'magit-status-mode)))
@@ -872,13 +872,5 @@ as argument."
                                        (pop-to-buffer buf))))))
                              process))))))
 
-;;; magit-process.el ends soon
-
-(define-obsolete-variable-alias 'magit-log-output-coding-system
-  'magit-git-output-coding-system "Magit 2.9.0")
-
 (provide 'magit-process)
-;; Local Variables:
-;; indent-tabs-mode: nil
-;; End:
 ;;; magit-process.el ends here
